module Interviews.Helpers exposing (..)

import Config.Colour exposing (..)
import Config.Format exposing (..)
import Cuckery.Types exposing (..)
import Effect exposing (Effect)
import Element as E exposing (..)
import Element.Background as B exposing (..)
import Element.Border as D
import Element.Font as F
import Html.Attributes as H exposing (style)
import Interviews.Types exposing (..)
import Layouts
import Page exposing (Page)
import Route exposing (Route)
import Shared
import View exposing (View)


interviewMaker : Interview -> Element msg
interviewMaker interview =
    row
        [ spacing 20
        , width fill
        , E.height fill
        , alignTop
        , alignRight
        ]
        [ imageMaker interview
        , column
            [ E.width <| px 650 ]
            [ nameMaker interview
            , column
                [ E.height fill
                , E.width fill
                , B.color colourTheme.cardBackground
                , paddingEach
                    { top = 10
                    , bottom = 10
                    , left = 10
                    , right = 10
                    }
                , D.roundEach
                    { topLeft = 0
                    , topRight = 0
                    , bottomRight = 26
                    , bottomLeft = 26
                    }
                , spacing 3
                ]
                [ column
                    [ paddingEach
                        { top = 0
                        , bottom = 0
                        , left = 15
                        , right = 15
                        }
                    ]
                    [ column
                        [ spacing 8
                        ]
                        [ socialMaker interview
                        , appearanceTitle interview
                        , appearanceMaker interview
                        ]
                    ]
                ]
            ]
        ]


imageMaker : Interview -> Element msg
imageMaker interview =
    column
        [ E.width <| px 115
        , E.height <| px 115
        , alignTop
        , alignRight
        ]
        [ column
            [ D.rounded 100
            , D.width 5
            , D.color colourTheme.cardBackground
            , B.color colourTheme.cardBackground
            ]
            [ E.image
                [ alignRight
                , alignTop
                , D.rounded 100
                , clip
                , E.width <| px 90
                , E.height <| px 90
                ]
                { src = "interviews/" ++ interview.interviewImage ++ ".png"
                , description = interview.interviewImage
                }
            ]
        ]


nameMaker : Interview -> Element msg
nameMaker interview =
    row
        (nonHighlightedTitleFormat
            ++ [ F.size 20
               , paddingEach
                    { top = 6
                    , bottom = 3
                    , left = 25
                    , right = 15
                    }
               , alignBottom
               , width fill
               , B.color colourTheme.highlightTextHover
               , D.roundEach
                    { topLeft = 26
                    , topRight = 26
                    , bottomRight = 0
                    , bottomLeft = 0
                    }
               ]
        )
        [ text interview.interviewName ]


socialMaker : Interview -> Element msg
socialMaker interview =
    row
        (paragraphBoldFormat
            ++ [ F.size 18
               , spacing 5
               ]
        )
        [ text "Social:"
        , paragraphLinkFormat
            { url = interview.interviewSocial
            , label = transitionHighlightedLinkHover <| text (formatInterviewSocial interview.interviewSocial)
            }
        ]


appearanceTitle : Interview -> Element msg
appearanceTitle interview =
    row
        (paragraphBoldFormat
            ++ [ F.size 18
               ]
        )
        [ text "Appearances:" ]


appearanceMaker : Interview -> Element msg
appearanceMaker interview =
    column [ spacing 15, width fill ] <|
        List.map2 (\x y -> makeAppearance x y)
            interview.interviewAppearances
            (List.range 1 (List.length interview.interviewAppearances))


makeAppearance : Appearance -> Int -> Element msg
makeAppearance appearanceEntry index =
    column
        (paragraphAlignLeft
            ++ [ spacing 8
               , width fill
               ]
        )
        [ appearanceEpisode appearanceEntry index
        , subjectMaker appearanceEntry index
        , subjectList appearanceEntry
        ]


appearanceEpisode : Appearance -> Int -> Element msg
appearanceEpisode appearanceEntry index =
    row
        (paragraphFormat
            ++ [ F.size 18
               , E.width fill
               , paddingEach
                    { top = 0
                    , bottom = 0
                    , left = 35
                    , right = 0
                    }
               ]
        )
        [ text (String.fromInt index ++ ". ")
        , paragraphLinkFormat
            { url = appearanceEntry.appearanceLink
            , label =
                row
                    [ F.size 18
                    ]
                    [ text ("#" ++ appearanceEntry.appearanceEpisode ++ ": " ++ appearanceEntry.appearanceTitle)
                        |> el
                            [ F.color colourTheme.highlightText
                            , mouseOver [ F.color colourTheme.highlightTextHover ]
                            , transitionStyle
                            ]
                    ]
            }
        ]


subjectMaker : Appearance -> Int -> Element msg
subjectMaker appearanceEntry index =
    row paragraphBoldFormat
        [ column
            [ alignTop
            , paddingEach
                { top = 0
                , right = 0
                , bottom = 0
                , left = 55
                }
            ]
            [ text "Subjects:"
            ]
        ]


subjectList : Appearance -> Element msg
subjectList appearanceEntry =
    column
        [ spacing 8
        , width fill
        , paddingEach
            { top = 0
            , right = 0
            , bottom = 0
            , left = 75
            }
        ]
    <|
        List.map2 (\x y -> makeSubject x)
            appearanceEntry.appearanceSubjects
            (List.range 1 (List.length appearanceEntry.appearanceSubjects))


makeSubject : Subjects -> Element msg
makeSubject subjects =
    column
        [ E.width fill
        , alignLeft
        , paddingEach
            { top = 0
            , right = 0
            , bottom = 0
            , left = 8
            }
        ]
        [ paragraph [ F.regular ]
            [ text ("‣ " ++ subjects.subject) ]
        ]
