module Services.Helpers exposing (..)

import Config.Colour exposing (..)
import Config.Format exposing (..)
import Effect exposing (Effect)
import Element as E exposing (..)
import Element.Background as B exposing (..)
import Element.Border as D
import Element.Font as F
import Html.Attributes as H exposing (style)
import Layouts
import Page exposing (Page)
import Route exposing (Route)
import Services.ServiceList.Creative.ElmBuilds exposing (servicesElmBuilds)
import Services.Types exposing (..)
import Shared
import View exposing (View)


serviceMaker : Service -> Element msg
serviceMaker service =
    row
        [ spacing 20
        , E.width fill
        , E.height fill
        , E.alignTop
        , E.alignRight
        ]
        [ imageMaker service
        , column
            [ E.width <| px 650 ]
            [ titleMaker service
            , column
                [ E.height fill
                , E.width fill
                , B.color colourTheme.cardBackground
                , paddingEach
                    { top = 10
                    , bottom = 10
                    , left = 10
                    , right = 10
                    }
                , D.roundEach
                    { topLeft = 0
                    , topRight = 0
                    , bottomRight = 26
                    , bottomLeft = 26
                    }
                , spacing 3
                ]
                [ column
                    [ paddingEach
                        { top = 0
                        , bottom = 0
                        , left = 15
                        , right = 15
                        }
                    , spacing 8
                    ]
                    [ rateMaker service
                    , descriptionMaker service
                    , offeringMaker service
                    ]
                ]
            ]
        ]


imageMaker : Service -> Element msg
imageMaker service =
    column
        [ E.width <| px 115
        , E.height <| px 115
        , E.alignTop
        , E.alignRight
        ]
        [ column
            [ D.rounded 100
            , D.width 5
            , D.color colourTheme.cardBackground
            , B.color colourTheme.cardBackground
            ]
            [ E.image
                [ E.alignRight
                , alignTop
                , D.rounded 100
                , clip
                , E.width <| px 90
                , E.height <| px 90
                ]
                { src = "services/" ++ service.serviceImage ++ ".png"
                , description = service.serviceName
                }
            ]
        ]


titleMaker : Service -> Element msg
titleMaker service =
    row
        (nonHighlightedTitleFormat
            ++ [ F.size 20
               , B.color colourTheme.highlightTextHover
               , paddingEach
                    { top = 6
                    , bottom = 3
                    , left = 25
                    , right = 15
                    }
               , alignBottom
               , E.width fill
               , D.roundEach
                    { topLeft = 26
                    , topRight = 26
                    , bottomRight = 0
                    , bottomLeft = 0
                    }
               ]
        )
        [ text service.serviceName
        ]


serviceWidth =
    width <| px 80


rateMaker : Service -> Element msg
rateMaker service =
    row
        (paragraphBoldFormat
            ++ [ F.size 18
               , E.width fill
               ]
        )
        [ column
            [ alignTop
            , serviceWidth
            ]
            [ text "Rate:"
            ]
        , column
            [ E.width fill
            , alignLeft
            ]
            [ paragraph [ F.regular ]
                [ el [ F.color colourTheme.highlightText ] <|
                    text service.serviceRate
                ]
            ]
        ]


descriptionMaker : Service -> Element msg
descriptionMaker service =
    row paragraphBoldFormat
        [ column
            [ alignTop
            , width <| px 80
            ]
            [ text "Offerings:"
            ]
        ]


offeringMaker : Service -> Element msg
offeringMaker service =
    column
        [ spacing 8
        , width fill
        , paddingEach
            { top = 0
            , right = 0
            , bottom = 0
            , left = 35
            }
        ]
    <|
        List.map2 (\x y -> makeDescription x)
            service.serviceDescription
            (List.range 1 (List.length service.serviceDescription))


makeDescription : Description -> Element msg
makeDescription description =
    column
        (paragraphAlignLeft
            ++ [ spacing 8
               , width fill
               ]
        )
        [ paragraph [ F.regular ]
            [ text ("‣ " ++ description.point) ]
        ]
